unit hiIconAnimation; {       ver 1.30 }

interface

uses Windows,Share,Debug,Kol;

type
  ThiIconAnimation = class(TDebug)
   private
    FListIndex: PStrListEx;
    FListBitmap: PImageList;
    FIcon: PIcon;
    FBitmap: PBitmap; 
    FTimer: PTimer;
    FTimerCycle: PTimer;
    FDelay: integer;
    FDelayCycle: integer;
    FImgSize: Integer;
    FCircleAnima: boolean;

    procedure _OnTimer(Obj:PObj);
    procedure _OnTimerCycle(Obj:PObj);          
    procedure SetIndex(Value:PStrListEx);
    procedure SetBitmap(Value:PStrListEx);    
    procedure SetImgSize(Value:integer);
   public

    _event_onIconFrame: THI_event;
    _event_onBitmapFrame: THI_event;
    _event_onEndCycle: THI_event;    

    _data_Index: THI_event;

    _prop_MaskColor : integer;
    _prop_TranspColor : integer;
    _prop_Index       : integer;

    property _prop_IndexArray  : PStrListEx write SetIndex;    
    property _prop_Bitmaps     : PStrListEx write SetBitmap;
    property _prop_ImgSize     : integer write SetImgSize;
    property _prop_Delay       : integer read FDelay write FDelay;
    property _prop_DelayCycle  : integer read FDelayCycle write FDelayCycle;
    property _prop_CircleAnima : boolean read FCircleAnima write FCircleAnima;

    constructor Create;
    destructor Destroy; override;

    procedure _work_doAnimation(var _Data:TData; Index:word);
    procedure _work_doIndexFrame(var _Data:TData; Index:word);
    procedure _work_doStop(var _Data:TData; Index:word);
    procedure _work_doDelay(var _Data:TData; Index:word);
    procedure _work_doDelayCycle(var _Data:TData; Index:word);
    procedure _work_doCircleAnima(var _Data:TData; Index:word);

  end;

var
   FStop     : boolean = false;
   FFlagDelay: boolean = false;   
   FStopCycle: boolean = false;
   FCurFrame : integer = 0;
   FCurIndex : integer = 0;
   
implementation

constructor ThiIconAnimation.Create;
begin
   inherited;
   FIcon := NewIcon;
   FBitmap := NewBitmap(0,0);

   FTimer := NewTimer(83);
   FTimer.Enabled := false;
   FTimer.OnTimer := _onTimer;

   FTimerCycle := NewTimer(83);
   FTimerCycle.Enabled := false;
   FTimerCycle.OnTimer := _onTimerCycle;
   
end;

destructor ThiIconAnimation.Destroy;
begin
   FTimer.free;
   FTimerCycle.free;
   FIcon.free;
   FBitmap.free;
   if Assigned(FListIndex) then FListIndex.free; 
   if Assigned(FListBitmap) then FListBitmap.free;
   inherited;
end;

procedure ThiIconAnimation._OnTimer;
var      db,di:TData;
begin
   if FFlagDelay then exit;
   if FStop then begin
      FFlagDelay := false;
      FTimer.Enabled := false;
      FTimerCycle.Enabled := false;
      exit;
   end;

   if not (Assigned(FListIndex) and Assigned(FListBitmap)) then exit;

   if (FCurIndex >= 0) and (FCurIndex < FListIndex.Count) then begin   
      FCurFrame := Integer(FListIndex.Objects[FCurIndex]);
      if (FCurFrame >= 0) and (FCurFrame < FListBitmap.Count) then begin
         FIcon.Clear;
         FIcon.Handle := FListBitmap.ExtractIcon(FCurFrame);
      end;
   end;

   if FIcon.Handle = 0 then exit;
    
   FBitmap.Clear;
   FBitmap.Handle := FIcon.Convert2Bitmap(_prop_TranspColor);

   dtBitmap(db,FBitmap);
   dtIcon(di,FIcon);
   inc(FCurIndex);
   if FCurIndex > FListIndex.Count - 1 then begin
      FCurIndex := 0;
      if FCircleAnima then
         if FStopCycle then begin
            FStop := true;
            FFlagDelay := false; 
          end else begin
            FFlagDelay := true;
            FTimerCycle.Enabled := true;
      end else
         FStop := true;
      _hi_OnEvent(_event_onEndCycle);
   end;
   _hi_OnEvent(_event_onIconFrame, di);
   _hi_OnEvent(_event_onBitmapFrame, db);
end;

procedure ThiIconAnimation._OnTimerCycle;
begin
   FFlagDelay := false;
   FTimerCycle.Enabled := false;
   if FStopCycle then FTimer.Enabled := false;
end;

procedure ThiIconAnimation.SetIndex;
begin
   FListIndex := NewStrListEx;
   FListIndex := Value;
end;

procedure ThiIconAnimation.SetImgSize;
begin
   FImgsize := Value;
   if FImgsize < 16 then FImgsize:= GetSystemMetrics(SM_CXICON);
end;

procedure ThiIconAnimation.SetBitmap;
var   Icon: PIcon;
      IconInfo: TIconInfo;
      bmp,mask,body: PBitmap;
      i: integer;      
begin
   FListBitmap := NewImageList(nil);
   for i := 0 to Value.Count - 1 do begin 
      FListBitmap.ImgWidth := FImgSize;
      FListBitmap.ImgHeight := FImgSize;
      FListBitmap.DrawingStyle := [dsTransparent];
      
      bmp := NewBitmap(FImgSize,FImgSize);
      bmp.Handle := Value.Objects[i];
      body := NewBitmap(0,0);
      if bmp.Handle <> 0 then body.Assign(bmp);
      mask := NewBitmap(0,0);
      mask.Assign(body);
      if not mask.Empty then mask.Convert2Mask(Color2RGB(_prop_MaskColor));

      IconInfo.fIcon := true;
      IconInfo.xHotspot := 0;
      IconInfo.yHotspot := 0;
      IconInfo.hbmMask := mask.Handle;
      IconInfo.hbmColor := body.Handle;
      Icon := NewIcon;
      Icon.Handle := CreateIconIndirect(IconInfo);
      FListBitmap.AddIcon(Icon.Handle);
      mask.Free;
      body.Free;   
      bmp.free;
      Icon.free;
   end;
end;

procedure ThiIconAnimation._work_doAnimation;
begin
   FTimer.Interval := FDelay;
   FTimerCycle.Interval := FDelayCycle;
   FCurIndex := ReadInteger(_Data, _data_Index, _prop_Index);
   FFlagDelay := false;
   FStopCycle := false;
   FStop := false;
   _OnTimer(nil);
   FTimer.Enabled := true;
end;

procedure ThiIconAnimation._work_doStop;
begin
   if FCircleAnima then FStopCycle := true;
end;

procedure ThiIconAnimation._work_doIndexFrame;
begin
   FCurIndex := ReadInteger(_Data, _data_Index, _prop_Index);
   FStop := false;
   _OnTimer(nil);
end;

procedure ThiIconAnimation._work_doDelay;
begin
   FDelay := ToInteger(_Data);
   if FDelay < 10 then FDelay := 10;  
   FTimer.Interval := FDelay;
end;

procedure ThiIconAnimation._work_doDelayCycle;
begin
   FDelayCycle := ToInteger(_Data);
   if FDelayCycle < 10 then FDelayCycle := 10;
   FTimerCycle.Interval := FDelayCycle;
end;

procedure ThiIconAnimation._work_doCircleAnima; begin FCircleAnima := ReadBool(_Data); end;

end.